/******************************************************************************
 * $Header: /CommonBe/agmsmith/Programming/SteadyCast/KernelLeakTest/RCS/KernelLeaker.cpp,v 1.2 2019/11/22 21:42:23 agmsmith Exp $
 *
 * Compile with:  g++ -lbe KernelLeaker.cpp
 *
 * Looking in to a kernel_team memory leak - seems to happen when BMessages are
 * sent to different teams when using the same BMessenger.  Run it and watch
 * the memory usage in ProcessController for the kernel_team, you'll see it
 * cranking up slowly.  When the program is exited, the leaks remain.
 *
 * By AGMS, write to agmsmith@ncf.ca
 */

/* Standard C Library. */

#include <stdio.h>
#include <stdlib.h>

/* BeOS (Be Operating System) headers. */

#include <Application.h>

static const char *g_AppSignature =
  "application/x-vnd.agmsmith.kernel-leak-20191122";


class KernelLeakerApp : public BApplication
{
public:
  /* Constructor and destructor. */
  KernelLeakerApp ();
  ~KernelLeakerApp ();

  /* BeOS virtual functions. */
  virtual void Pulse ();
  virtual bool QuitRequested ();
  virtual void ReadyToRun ();

};


/******************************************************************************
 * Implementation of the KernelLeakerApp class.  Constructor, destructor and
 * the rest of the member functions in mostly alphabetical order.
 */

KernelLeakerApp::KernelLeakerApp () : BApplication (g_AppSignature)
{
  SetName ("Kernel Leaker $Revision: 1.2 $");
}


KernelLeakerApp::~KernelLeakerApp ()
{
}


void KernelLeakerApp::Pulse ()
{
  status_t ErrorCode = B_OK;
  BMessage ReplyMessage;
  BMessage ScriptMessage;

  static const char *Signatures[] = { // Deskbar and Tracker.
    "application/x-vnd.Be-TSKB",
    "application/x-vnd.Be-TRAK",
    "application/x-vnd.Be-TSKB",
    "application/x-vnd.Be-TRAK",
    "application/x-vnd.Be-TSKB",
    "application/x-vnd.Be-TRAK",
    "application/x-vnd.Be-TSKB",
    "application/x-vnd.Be-TRAK",
    "application/x-vnd.Be-TSKB",
    "application/x-vnd.Be-TRAK",
    NULL
  };

  int iSig;
  for (iSig = 0; Signatures[iSig] != NULL; iSig++)
  {
    ErrorCode = B_OK;
    BMessenger MessengerToTarget(Signatures[iSig], -1, &ErrorCode);
    if (ErrorCode != B_OK)
    {
      printf ("Failed to set target.\n");
      return;
    }

    // Send BMessages to various loopers in the target application.

    int i;
    for (i = 0; i < 10; i++)
    {
      ScriptMessage.MakeEmpty ();
      ScriptMessage.what = B_GET_PROPERTY;
      ScriptMessage.AddSpecifier ("InternalName");
      ScriptMessage.AddSpecifier ("Looper", (int32) i);
      ReplyMessage.MakeEmpty ();
      ErrorCode = MessengerToTarget.SendMessage (
        &ScriptMessage, &ReplyMessage,
        100000 /* delivery timeout */, 100000 /* reply timeout */);
      const char *NameString = NULL;
      if (ErrorCode != B_OK || ReplyMessage.FindString ("result",
      &NameString) != B_OK || NameString == NULL)
        continue; // Ignore Loopers that don't exist or aren't scriptable.

      // printf ("Name %d: %s\n", i, NameString);
    }
  }
}


bool KernelLeakerApp::QuitRequested ()
{
  return BApplication::QuitRequested ();
}


void KernelLeakerApp::ReadyToRun ()
{
  // Turn on the periodic Pulse() callback, maximum speed is 1/10 of a second,
  // though that's too fast (can't process quit messages).

  SetPulseRate (1500000);
}


/******************************************************************************
 * Finally, the main program which drives it all.
 */

int main (int argc, char**)
{
  KernelLeakerApp MyApp;

  if (MyApp.InitCheck () == 0)
  {
    MyApp.Run ();
  }
  printf ("Finished running the Kernel Leaker experiment.\n");
  return 0;
}


/******************************************************************************
 * $Log: KernelLeaker.cpp,v $
 * Revision 1.2  2019/11/22 21:42:23  agmsmith
 * Adapted for BeOS too (no BMessenger::SetTo() function).
 *
 * Revision 1.1  2019/11/22 21:34:59  agmsmith
 * Initial revision
 */
